/*
 * LetterOrder - Experimenting with the order of letters in words
 * Copyright (C) 2011-2015 Christian Schenk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.christianschenk.letterorder;

import java.util.List;

import org.christianschenk.letterorder.utils.LetterOrderUtils;
import org.christianschenk.letterorder.utils.TextUtils;
import org.christianschenk.letterorder.utils.enums.WordDistance;

import com.google.inject.Inject;

/**
 * Extracts the words from a text corpus and displays one possible variant with
 * obfuscated words.
 *
 * @author Christian Schenk
 */
public class LetterOrder {

	private final TextUtils textUtils;
	private final LetterOrderUtils letterOrderUtils;

	@Inject
	public LetterOrder(final TextUtils textUtils, final LetterOrderUtils letterOrderUtils) {
		this.textUtils = textUtils;
		this.letterOrderUtils = letterOrderUtils;
	}

	/**
	 * Returns one possible variant of the given text based on the supplied
	 * distance.
	 *
	 * @param text
	 *            some text
	 * @param wordDistance
	 *            defines how hard the words will be to read
	 * @return
	 */
	public String getTextVariant(final String text, final WordDistance wordDistance) {
		// Extract words
		final List<String> words = this.textUtils.getWords(text);

		// Get list with obfuscated words
		final List<String> obfuscatedWords = this.letterOrderUtils.getObfuscatedWords(words, wordDistance);

		// Compile string
		// TODO Restore removed characters from the original text.
		final StringBuilder buffer = new StringBuilder();
		for (final String word : obfuscatedWords) {
			buffer.append(word).append(" ");
		}
		return buffer.toString();
	}
}