/*
 * LetterOrder - Experimenting with the order of letters in words
 * Copyright (C) 2011-2015 Christian Schenk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.christianschenk.letterorder.utils;

/**
 * Helps calculating the distance between strings.
 *
 * XXX: Maybe include case insensitive comparison.
 *
 * @author Christian Schenk
 */
public class StringDistanceUtils {

	/**
	 * Returns the hamming distance.
	 */
	public int getHammingDistance(final String str1, final String str2) {
		if (str1 == null || str2 == null) throw new IllegalArgumentException("Input strings must not be null.");
		if (str1.length() != str2.length()) throw new IllegalArgumentException("Strings must be of equal lenght.");

		int distance = 0;
		for (int i = 0, n = str1.length(); i < n; i++)
			if (str1.charAt(i) != str2.charAt(i)) distance++;

		return distance;
	}

	/**
	 * Retruns the Levenshtein distance.
	 *
	 * Almost copied from here:
	 *
	 * <pre>
	 * http://en.wikibooks.org/wiki/Algorithm_Implementation/Strings/Levenshtein_distance#Java
	 * </pre>
	 */
	public int getLevenshteinDistance(final CharSequence str1, final CharSequence str2) {
		if (str1 == null || str2 == null) throw new IllegalArgumentException("Input strings must not be null.");

		final int[][] distance = new int[str1.length() + 1][str2.length() + 1];

		for (int i = 0; i <= str1.length(); i++)
			distance[i][0] = i;
		for (int j = 0; j <= str2.length(); j++)
			distance[0][j] = j;

		for (int i = 1; i <= str1.length(); i++)
			for (int j = 1; j <= str2.length(); j++)
				distance[i][j] = this.minimum(distance[i - 1][j] + 1, distance[i][j - 1] + 1, distance[i - 1][j - 1] + ((str1.charAt(i - 1) == str2.charAt(j - 1)) ? 0 : 1));

		return distance[str1.length()][str2.length()];
	}

	private int minimum(final int a, final int b, final int c) {
		return Math.min(Math.min(a, b), c);
	}
}