/*
 * LetterOrder - Experimenting with the order of letters in words
 * Copyright (C) 2011-2015 Christian Schenk
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.christianschenk.letterorder.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Utility methods for text processing.
 *
 * @author Christian Schenk
 */
public class TextUtils {

	/** Can be used to match at least German and English words */
	private static final Pattern WORD_CHARACTERS = Pattern.compile("[\\w'üöäß]+", Pattern.CASE_INSENSITIVE);

	/**
	 * Extracts all words from the given text.
	 *
	 * @param text
	 *            a certain text
	 * @return list of words
	 */
	public List<String> getWords(final String text) {
		final List<String> words = new ArrayList<String>();
		if (text == null || text.trim().isEmpty()) return words;

		// extract words
		final Matcher m = WORD_CHARACTERS.matcher(text);
		while (m.find())
			words.add(text.substring(m.start(), m.end()));

		return words;
	}
}